/*utils.js
description : visualize and explore firsttuesday's Web archives
licence : AGPL + CECILL v3
author : quentin lobbé - qlobbe@iscpif.fr*/

function readJson(file, callback) {
    var raw = new XMLHttpRequest();
    raw.overrideMimeType("application/json");
    raw.open("GET", file, true);
    raw.onreadystatechange = function() {
    	if (raw.readyState === 4 && raw.status == "200") {
        	callback(raw.responseText);
      	}
    }
    raw.send(null);
}

function timestampToDate(time) {
    let month_idx = parseInt(time.substring(4,6)) - 1
    return new Date(time.substring(0,4),month_idx,time.substring(6,8))
}

function getX(x0,radius,alpha) {
    return  x0 + radius * Math.sin(Math.PI * 2 * alpha / 360);
}

function getY(y0,radius,alpha) {
    return  y0 + radius * Math.cos(Math.PI * 2 * alpha / 360);
}

function getRadialRange(distribution,max_radius,min_radius) {
    
    let total  = Object.values(distribution).reduce((acc,cur)=> acc + cur,0),
        values = Object.values(distribution);
    
    var sup = max_radius,
        acc = 0,
        res = [];
    
    for (var i = 0; i < values.length; i++) {
        acc = acc + values[i];
        res.push((acc * sup / total) + min_radius);
    }
    
    return res;
}


function draw(data) {

    /* sub-functions */

    function halfway(stamp) {
        return (scale(timestampToDate(stamp[0])) + scale(timestampToDate(stamp[1]))) / 2
    }

    function captureOver(d,i) {
        let stamp = d3.select(this).attr("stamp")
        let url = d3.select(this).attr("url")
        document.getElementById("hover_time").innerHTML = timestampToDate(stamp).toDateString()
        document.getElementById("hover_url").innerHTML = "https://web.archive.org/web/" + stamp + "/https://" + url 
    }

    function cerneOver(d,i) {
        let stamp = d3.select(this).attr("stamp")
        document.getElementById("hover_time").innerHTML = timestampToDate(stamp).toDateString()
        document.getElementById("hover_url").innerHTML = ""
    } 

    function momentOver(d,i) {
        let from = d3.select(this).attr("stamp_from")
        let to = d3.select(this).attr("stamp_to")
        let url = d3.select(this).attr("url")
        document.getElementById("hover_time").innerHTML = " from " + timestampToDate(from).toDateString() + " to " + timestampToDate(to).toDateString()
        document.getElementById("hover_url").innerHTML = "https://web.archive.org/web/" + from + "/https://" + url
    }        

    function drawCapture(stamp,status,community,alpha,url) {
        let r = scale(timestampToDate(stamp)),
            x = getX(x0,r,alpha),
            y = getY(y0,r,alpha),
            cls = ((status > 0) ? "dot-archived archived" : "dot-unarchived un-archived");
        panel
          .append("circle")
          .attr("class",cls)
          .attr("stamp",stamp)
          .attr("url",url)
          .attr("cx", x)
          .attr("cy", y)
          .attr("r" , 1)
          .on("mouseover", captureOver);             
    }    

    function drawCliques() {
        for (const [k, v] of Object.entries(data.cliques)) {
            
            let a1 = v.alpha[0],
                a2 = v.alpha[1];

            let r1 = halfway(v.stamps[0]),
                r2 = halfway(v.stamps[1]);

            let x1 = getX(x0,r1,a1),
                x2 = getX(x0,r2,a2),
                y1 = getY(y0,r1,a1),
                y2 = getY(y0,r2,a2);

            panel
                .append("line")
                .attr("class","clique")
                .attr("id", v.moments[0] + '-' + v.moments[1])
                .attr("x1", x1)
                .attr("y1", y1)
                .attr("x2", x2)
                .attr("y2", y2)
        }        
    }

    function onZoom() {
        panel.selectAll("path").attr("transform", d3.event.transform);
        panel.selectAll("line").attr("transform", d3.event.transform);
        panel.selectAll("circle").attr("transform", d3.event.transform);
    }

    function onZoomStart() {
        d3.selectAll(".clique").remove()
    }

    function onZoomEnd() {
        drawCliques()
        d3.selectAll(".clique").attr("transform", d3.event.transform);
        if (! document.getElementById('cliques').checked) {
            [].forEach.call(document.querySelectorAll('.clique'), function (el) {
              el.style.visibility = 'hidden';
            });
        }
    }

    function resetZoom() {
      svg.transition()
          .duration(750)
          .call(zoom.transform, d3.zoomIdentity);
    }     


    /* d3 svg */

    var div = d3.select('#sketch_div')
    			.node().getBoundingClientRect(),
          w = div.width,
          h = div.height; 

    const svg = d3
        .select('#sketch_div')
        .append("svg")
        .attr("viewBox",[0,0,w,h]);      
        
    const panel = svg.append("g");

    var max_date = data.to   + "0101000000",
        min_date = data.from + "0101000000",
        max_radius = h / 2 - h / 20,
        min_radius = h / 100,
        x0 = w/2,
        y0 = h/2,
        alpha_step = 360 / Object.keys(data.urls).length,
        alpha = 0,
        cpt = 0;

    /* d3 zoom */

    var zoom = d3.zoom()
        .scaleExtent([1,50])
        .extent([[0,0],[w,h]])
        .on("start", onZoomStart)
        .on("zoom", onZoom)
        .on("end", onZoomEnd)

    svg.call(zoom)

    d3.select("#reset").on("click",resetZoom);

    /* radial scale */

    var scaleDomain = [timestampToDate(min_date)].concat(Object.keys(data.distribution).map(x => timestampToDate(x + "0101000000")));
    var scaleRange  = [0].concat(getRadialRange(data.distribution,max_radius,min_radius));
    var scale = d3.scaleTime().domain(scaleDomain).range(scaleRange);

    /* colors */

    window.colors = {}
    var colorRange = d3.schemeCategory10;

    /* cernes */

    var lineGenerator = d3
        .line()
        .curve(d3.curveCardinal)
        .defined(function(d) {
            return d !== null;
        });

    alpha = 180;
    cpt = 0;

    Object.keys(data.distribution).forEach(function(k,idx,keys){
        
        // for each year
       
        let thr = k + "0101000000";

        if (idx != (keys.length - 1)) {
           
            // inner cerne

            panel
              .append('circle')
              .attr("class","cerne in-cerne")
              .attr("stamp",thr)
              .attr('cx', x0)
              .attr('cy', y0)
              .attr('r' , scale(timestampToDate(thr)))
              .on("mouseover", cerneOver);

        } else {

            // outer cerne

            let sequence = [];
            let points = [];
            let origin = [];

            previousStatus = 0;
            previousPoints = [];

            for (const [k, v] of Object.entries(data.urls)) {

                // for each url

                // set the colors 

                let community = v.community;
                if (!(community in window.colors)) {
                    if (Object.keys(window.colors).length > 10) {
                        let a = Math.floor(Object.keys(window.colors).length / 10)
                        window.colors[community] = colorRange[Object.keys(window.colors).length - a * 10]
                    } else {
                        window.colors[community] = colorRange[Object.keys(window.colors).length - 1]
                    }
                }


                // create a point for the outer cerne

                alpha += alpha_step;
                
                let status   = v.status
                let lstStamp = v.moments.slice(-1)[0][1][1];
                let r = scale(timestampToDate(lstStamp)) + min_radius;

                let x = getX(x0,r,alpha),
                    y = getY(y0,r,alpha);
                if (points.length == 0) {
                    // to have a closed curve
                    origin = [x,y];
                    previousStatus = status;
                }

                if (previousStatus != status) {
                    sequence.push({'status':previousStatus,'points':points})
                    points = [previousPoints]
                }  

                previousStatus = status;
                previousPoints = [x,y];
                
                points.push([x,y]);  


            }

            points.push(origin); 
            sequence.push({'status':previousStatus,'points':points})

            sequence.forEach((segment) => {
                let line = lineGenerator(segment.points);
                let cls  = ((segment.status > 0) ? "out-cerne-archived" : "out-cerne-un-archived");
                panel
                  .append('path')
                  .attr("class",cls)
                  .attr("stamp",thr)
                  .attr('d', line)
                  .on("mouseover", cerneOver);
            })
        
        }

        cpt ++;
        
    })

    /* draw the moments */

    alpha = 180;

    for (const [k, v] of Object.entries(data.urls)) {
         
        alpha += alpha_step;

        var size = v.moments.length;
        var community = v.community;

        let inf = v.moments[0][1][0],
            sup = v.moments[size -1][1][1];

        let r1 = scale(timestampToDate(inf)),
            r2 = scale(timestampToDate(sup)),
            x1 = getX(x0,r1,alpha),
            y1 = getY(y0,r1,alpha),
            x2 = getX(x0,r2,alpha),
            y2 = getY(y0,r2,alpha);

        panel
            .append("line")
            .attr("class","moment")
            .attr("stamp_from",inf)
            .attr("stamp_to",sup)
            .attr("community",community)
            .style("stroke", "#0d1824") 
            //.style("stroke", window.colors[community]) 
            .attr("url",k)
            .attr("x1", x1)
            .attr("y1", y1)
            .attr("x2", x2)
            .attr("y2", y2)
            .on("mouseover", momentOver);  
        
    }     

    /* captures */

    alpha = 180;

    for (const [k, v] of Object.entries(data.urls)) {
        alpha += alpha_step;
        let status = v.status
        let community = v.community;
        v.moments.forEach(function(moment){
            if (moment[1][0] == moment[1][1]) {
                drawCapture(moment[1][0],status,community,alpha,k)
            }
        })
    }

    /* cliques */

    drawCliques();  

}